package org.rcisoft.business.opmArticle.service.impl;


import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.baidu.censor.dto.CensorResult;
import org.rcisoft.baidu.censor.service.ContentCensorService;
import org.rcisoft.business.cmsOrder.entity.CmsOrder;
import org.rcisoft.business.memGoldCoinFlow.dao.MemGoldCoinFlowRepository;
import org.rcisoft.business.memGoldCoinFlow.entity.MemGoldCoinFlow;
import org.rcisoft.business.memInfo.bean.MemberInfoRedisBean;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.memInfo.entity.MemInfo;
import org.rcisoft.business.memInfo.service.impl.MemInfoServiceImpl;
import org.rcisoft.business.opmArticle.bean.OpmArticleRedisBean;
import org.rcisoft.business.opmArticle.dao.OpmArticleRepository;
import org.rcisoft.business.opmArticle.entity.ArticleCommentDTO;
import org.rcisoft.business.opmArticle.entity.ArticleCommentVO;
import org.rcisoft.business.opmArticle.entity.LikeDTO;
import org.rcisoft.business.opmArticle.entity.OpmArticle;
import org.rcisoft.business.opmArticle.service.OpmArticleService;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.service.CyFileStorageService;
import org.rcisoft.core.service.impl.CyRedisServiceImpl;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.util.TimeUtil;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.dictionary.entity.DictData;
import org.rcisoft.sys.dictionary.service.DictionaryService;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigInteger;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * Created by cy on 2024年3月30日 下午1:51:15.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class OpmArticleServiceImpl extends ServiceImpl<OpmArticleRepository, OpmArticle> implements OpmArticleService {

    @Autowired
    private DictionaryService dictionaryService;
    @Autowired
    private ContentCensorService contentCensorService;
    @Autowired
    private CyFileStorageService cyFileStorageService;
    @Autowired
    private MemInfoRepository memInfoRepository;
    @Autowired
    private CyRedisServiceImpl cyRedisServiceImpl;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private MemGoldCoinFlowRepository memGoldCoinFlowRepository;
    @Autowired
    private UserUtil userUtil;
    /**
    * 保存 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(OpmArticle opmArticle){
        String userId = CyUserUtil.getAuthenBusinessId();
        //1、实名判断
        userUtil.checkUserAuth("请先实名认证!");
        //2、判断今日发布动态的数量限制
        //获取当天时间 例2025/2/20 后获取0220
        LocalDate today = LocalDate.now();
        DateTimeFormatter monthFormatter = DateTimeFormatter.ofPattern("MM");
        DateTimeFormatter dayFormatter = DateTimeFormatter.ofPattern("dd");
        // 格式化并获取月份和日期
        String month = today.format(monthFormatter);
        String day = today.format(dayFormatter);
        String date = month + day;
        List<DictData> articleConfig = dictionaryService.selectByTypes("article_config");
        DictData dictData1 = articleConfig.stream().filter(item -> item.getDictLabel().equals("publish_article_count")).findFirst().orElse(null);
        DictData dictData2 = articleConfig.stream().filter(item -> item.getDictLabel().equals("pay_publish_article_count")).findFirst().orElse(null);
        DictData dictData3 = articleConfig.stream().filter(item -> item.getDictLabel().equals("pay_count")).findFirst().orElse(null);
        Integer publishCount = Integer.valueOf(dictData1.getDictValue());//每天最多发a条动态（免费）
        Integer payPublishCount = Integer.valueOf(dictData2.getDictValue());//每天最多发b条动态（付费）
        Integer publishCountAll = publishCount + payPublishCount;//免费+付费总数
        Integer payCount = Integer.valueOf(dictData3.getDictValue());//发布付费动态需要花费的金币数
        Object redisCount = cyRedisServiceImpl.get(OpmArticleRedisBean.USER_ARTICLE_COUNT + date + ":" + userId);
        if (redisCount != null && (int)redisCount >= publishCountAll){
            //超出最大数量
            throw new CyServiceException(1001,"今日发布动态次数已达上限!");
        } else if (redisCount == null){
            //当天第一次发布动态
            cyRedisServiceImpl.set(OpmArticleRedisBean.USER_ARTICLE_COUNT + date + ":" + userId,1,48*60*60L);
        } else if (redisCount != null && (int)redisCount < publishCountAll){
            //没超出最大数量
            //判断免费和付费数量
            if ((int)redisCount >= publishCount){
                //超过免费限制次数 需要付费
                if (opmArticle.getIsPay().equals("1")){
                    //扣除该用户的金币数量
                    int line = 0;
                    boolean isGetLock = false;
                    RLock lock = redissonClient.getLock(OpmArticleRedisBean.USER_ARTICLE_LOCK + userId);
                    try {
                        isGetLock = lock.tryLock(Long.parseLong(OpmArticleRedisBean.USER_ARTICLE_LOCK_WAIT_TIME),
                                Long.parseLong(OpmArticleRedisBean.USER_ARTICLE_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                        if (isGetLock) {
                            MemInfo memInfo = memInfoRepository.selectByOpenId(opmArticle.getWxOpenid());
                            if (memInfo.getGoldCoinsCount() - payCount < 0){
                                throw new CyServiceException(1003,"金币不足,请先充值!");
                            }
                            memInfo.setGoldCoinsCount(memInfo.getGoldCoinsCount() - payCount);
                            memInfoRepository.updateById(memInfo);
                            //添加消费记录
                            MemGoldCoinFlow memGoldCoinFlow = new MemGoldCoinFlow();
                            memGoldCoinFlow.setType("2");
                            memGoldCoinFlow.setActionType("21");
                            memGoldCoinFlow.setCount(payCount);
                            memGoldCoinFlow.setEndCount(memInfo.getGoldCoinsCount());
                            memGoldCoinFlow.setTargetId(BigInteger.valueOf(Long.parseLong(userId)));
                            memGoldCoinFlowRepository.insert(memGoldCoinFlow);
                            //增加操作
                            redisTemplate.opsForValue().increment(OpmArticleRedisBean.USER_ARTICLE_COUNT + date + ":" + userId,1L);
                        }
                        lock.unlock();
                    } catch (InterruptedException e) {
                        Thread.currentThread().interrupt();
                        lock.unlock();
                        isGetLock = false;
                        throw new CyServiceException(1004,"服务器繁忙，请稍后再试");
                    }
                } else {
                    //不进行发表动态
                    throw new CyServiceException(1002,"今日发表动态次数已达上限!");
                }
            } else {
                //没超过免费次数限制
                //增加操作
                redisTemplate.opsForValue().increment(OpmArticleRedisBean.USER_ARTICLE_COUNT + date + ":" + userId,1L);
            }
        }
        //3、进行审核   从redis获取动态审核配置  0：自动审核，1：人工审核
        List<DictData> sysConfig = dictionaryService.selectByTypes("sys_config");
        String style = sysConfig.get(0).getDictValue();
        if ("0".equals(style)) {
            //调用百度，自动审核
            boolean flag = true;
            List<String> msg = new ArrayList<>();
            if (StringUtils.isNotBlank(opmArticle.getContent())) {
                CensorResult result = contentCensorService.getCommonTextCensorResult(opmArticle.getContent());
                flag = result.getIsPass();
                if (!flag)
                    msg.addAll(result.getErrorMsg());
            }
            if (StringUtils.isNotBlank(opmArticle.getPictureId())){
                JSONArray ids = JSONUtil.parseObj(opmArticle.getPictureId()).getJSONArray("id");
                for (Object id : ids) {
                    String url = cyFileStorageService.getOssUrl((Integer)id);
                    CensorResult imgResult = contentCensorService.getImageCensorResult(url);
                    if (!imgResult.getIsPass())
                        msg.addAll(imgResult.getErrorMsg());
                    flag = flag && imgResult.getIsPass();
                }
            }
            if (flag)
                opmArticle.setExamStatus("1");
            else {
                opmArticle.setExamStatus("3");
                opmArticle.setRemarks(msg.toString());
            }
        }
        //判断是否是普通动态
        if (opmArticle.getTopicId() == null){
            //普通动态
            opmArticle.setIsArticle("1");
        } else {
            opmArticle.setIsArticle("0");
        }
        int line = baseMapper.insert(opmArticle);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
                opmArticle.getBusinessId()+"的opmArticle管理信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(OpmArticle opmArticle,String type){
        if (type.equals("app")){
            //app端
            //校验要删除的动态是不是当前登录人的
            QueryWrapper<OpmArticle> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(OpmArticle::getBusinessId,opmArticle.getBusinessId());
            wrapper.lambda().eq(OpmArticle::getCreateBy,CyUserUtil.getAuthenBusinessId());
            List<OpmArticle> opmArticles = baseMapper.selectList(wrapper);
            if (opmArticles.size() > 0){
                //是当前登录人的
                //校验是否超出删除时间
                List<DictData> articleConfig = dictionaryService.selectByTypes("article_config");
                DictData dictData = articleConfig.stream().filter(item -> item.getDictLabel().equals("delete_article_limit")).findFirst().orElse(null);
                Integer limitDay = Integer.valueOf(dictData.getDictValue());//限制天数
                long articleTime = opmArticles.get(0).getCreateDate().getTime() + limitDay * 24 * 60 * 60 * 1000;
                long nowTime = new Date().getTime();
                if (articleTime > nowTime){
                    opmArticle.setDelFlag("1");
                    int line = baseMapper.deleteById(opmArticle);
                    return new CyPersistModel(line);
                } else {
                    //超出规定时间  不能删除
                    throw new CyServiceException(1002,"已超出规定时间,不能删除!");
                }
            } else {
                throw new CyServiceException(1001,"您没有权限删除此动态!");
            }
        } else {
            //小程序端
            int line = baseMapper.deleteById(opmArticle);
            log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
                    opmArticle.getBusinessId()+"的opmArticle管理信息");
            return new CyPersistModel(line);
        }
    }

    /**
    * 逻辑删除 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(OpmArticle opmArticle){
        opmArticle.setDeleted();
        int line = baseMapper.deleteById(opmArticle);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                    opmArticle.getBusinessId()+"的opmArticle管理信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(OpmArticle opmArticle){
        //先查询该动态
        QueryWrapper<OpmArticle> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(OpmArticle::getBusinessId,opmArticle.getBusinessId());
        List<OpmArticle> list = baseMapper.selectList(wrapper);
        if (list.size() > 0){
            OpmArticle article = list.get(0);
            article.setExamStatus(opmArticle.getExamStatus());//修改审核状态
            baseMapper.updateArticleCommentExamStatus(article);
            //查询该动态下的评论数
            List<ArticleCommentVO> comment = baseMapper.selectArticleCommentCount(article);
            //修改动态
            article.setCommentCount(comment.size());//修改评论数
            //驳回原因
            if (opmArticle.getRemarks()!= null){
                article.setRemarks(opmArticle.getRemarks());
            }
            int line = baseMapper.updateById(article);
            //修改动态所有评论审核状态
            //修改子表的评论数 给父级评论数+1
            for (ArticleCommentVO commentVo : comment) {
                if (commentVo.getParentId() != null){
                    //查询评论的子评论数量
                    Integer commentCount = baseMapper.selectArticleChildrenCommentCount(commentVo);
                    //修改评论的评论数量
                    commentVo.setCommentCount(commentCount);
                    baseMapper.updateCommentById(commentVo);
                }
            }
            return new CyPersistModel(line);
        } else {
            throw new CyServiceException(1001,"该动态已被用户删除!");
        }
    }

    /**
    * 根据id查询 opmArticle管理
    * @param articleId
    * @return
    */
    @Override
    public OpmArticle findById(Integer articleId,String type){
        if (type.equals("app")){
            String loginUserId = CyUserUtil.getAuthenBusinessId();
            //根据动态id 查询动态详情
            OpmArticle opmArticle = baseMapper.selectArticleDetail(articleId,loginUserId);
            if (opmArticle == null){
                //该动态 对方已删除
                throw new CyServiceException(1001,"该动态已被对方删除!");
            }
            //查询该动态的评论
            List<ArticleCommentVO> list = baseMapper.appSelectArticleComment(opmArticle.getBusinessId(),loginUserId);
            list.forEach(item -> {
                //查询该条评论下的子评论
                List<ArticleCommentVO> childrenCommentList = baseMapper.appSelectArticleChildrenComment(opmArticle.getBusinessId(),item.getBusinessId(),loginUserId);
                item.setChildrenCommentList(childrenCommentList);
            });
            opmArticle.setArticleCommentVOList(list);
            return opmArticle;
        } else {
            String loginUserId = CyUserUtil.getAuthenBusinessId();
            //根据动态id 查询动态详情
            OpmArticle opmArticle = baseMapper.selectArticleDetail(articleId,loginUserId);
            //查询该动态的评论
            List<ArticleCommentVO> list = baseMapper.selectArticleComment(opmArticle.getBusinessId(),loginUserId);
            list.forEach(item -> {
                //查询该条评论下的子评论
                List<ArticleCommentVO> childrenCommentList = baseMapper.selectArticleChildrenComment(opmArticle.getBusinessId(),item.getBusinessId(),loginUserId);
                item.setChildrenCommentList(childrenCommentList);
            });
            opmArticle.setArticleCommentVOList(list);
            return opmArticle;
        }
    }

    /**
    * 分页查询 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Override
    public IPage<OpmArticle> findAllByPagination(CyPageInfo<OpmArticle> paginationUtility,
                                                 OpmArticle opmArticle,String type){
        IPage<OpmArticle> result = null;
        if (type.equals("app")){
            //小程序
            if (opmArticle.getArticleType() != null && opmArticle.getArticleType().equals("0")){
                //关注
                //查询该用户关注对象的id集合
                List<Integer> list = memInfoRepository.queryFollowIdById(CyUserUtil.getAuthenBusinessId());
                if (list.isEmpty()){
                    result = paginationUtility;
                } else {
                    opmArticle.setFollowList(list);
                    if (StringUtils.isNotEmpty(CyUserUtil.getAuthenBusinessId())){
                        //设置当前登录人id 用来查询该用户对文章是否已点赞
                        opmArticle.setLoginUserId(CyUserUtil.getAuthenBusinessId());
                    }
                    result = baseMapper.appQueryFollowOpmArticlePaged(paginationUtility,opmArticle);
                }
            } else if(opmArticle.getArticleType() != null && opmArticle.getArticleType().equals("1")){
                //最近
                if (StringUtils.isNotEmpty(CyUserUtil.getAuthenBusinessId())){
                    //设置当前登录人id 用来查询该用户对文章是否已点赞
                    opmArticle.setLoginUserId(CyUserUtil.getAuthenBusinessId());
                }
                result = baseMapper.appQueryOpmArticlePaged(paginationUtility,opmArticle);
            } else {
                //话题
                if (StringUtils.isNotEmpty(CyUserUtil.getAuthenBusinessId())){
                    //设置当前登录人id 用来查询该用户对文章是否已点赞
                    opmArticle.setLoginUserId(CyUserUtil.getAuthenBusinessId());
                }
                result = baseMapper.appQueryTopicOpmArticlePaged(paginationUtility,opmArticle);
            }
        } else {
            //管理端
            result = baseMapper.queryOpmArticlePaged(paginationUtility,opmArticle);
        }
        return result;
    }


    /**
     * 分页查询某个人的动态列表
     * @param paginationUtility
     * @param opmArticle
     * @return
     */
    @Override
    public IPage<OpmArticle> findUserArticleByPagination(CyPageInfo<OpmArticle> paginationUtility, OpmArticle opmArticle) {
        //校验 未关注的只能查5条  已关注的只能查近一个月的
        String isFollowed = opmArticle.getIsFollowed();
        if (StringUtils.isNotEmpty(isFollowed) && isFollowed.equals("0")){
            //1.未关注
            List<DictData> articleConfig = dictionaryService.selectByTypes("article_config");
            DictData dictData = articleConfig.stream().filter(item -> item.getDictLabel().equals("no_follow_article_limit")).findFirst().orElse(null);
            Integer noFollowArticleLimit = Integer.valueOf(dictData.getDictValue());
            String loginUserId = CyUserUtil.getAuthenBusinessId();
            if (StringUtils.isNotEmpty(opmArticle.getUserId())){
                //设置查询用户 查询某个人的动态列表
                opmArticle.setUserId(opmArticle.getUserId());
            }
            if (StringUtils.isNotEmpty(loginUserId)){
                //设置当前登录人id 用来查询该用户对文章是否已点赞
                opmArticle.setLoginUserId(loginUserId);
            }
            //设置动态的审核状态  只能查看审核通过的
            opmArticle.setExamStatus("1");
            //设置分页参数  防止前端下滑再次调分页 前端根据total 判断是否还有数据
            IPage<OpmArticle> opmArticleIPage = baseMapper.queryUserArticlePaged(paginationUtility, opmArticle);
            if (opmArticleIPage.getTotal() > 0){
                // 如果查出来的数据大于0，则判断限制数量是大于10还是小于10 如果大于10.则取整，例如11则取20
                // 判断noFollowArticleLimit是否能整除10
                if (noFollowArticleLimit > 0 && noFollowArticleLimit < 10){
                    // 限制条数小于10
                    opmArticleIPage.setTotal(noFollowArticleLimit);
                } else {
                    // 限制条数大于10
                    if (noFollowArticleLimit % 10 == 0){
                        // 如果能整除，则取整
                        opmArticleIPage.setTotal(noFollowArticleLimit);
                    } else {
                        // 如果不能整除，则取整加一  例如11，取20
                        noFollowArticleLimit = ((noFollowArticleLimit / 10)+1)*10;
                        opmArticleIPage.setTotal(noFollowArticleLimit);
                    }
                }
            } else {
                opmArticleIPage.setTotal(0);
            }
            return opmArticleIPage;
        } else if(StringUtils.isNotEmpty(isFollowed) && isFollowed.equals("1")){
            //2.已关注
            List<DictData> articleConfig = dictionaryService.selectByTypes("article_config");
            DictData dictData = articleConfig.stream().filter(item -> item.getDictLabel().equals("follow_article_limit")).findFirst().orElse(null);
            Integer followArticleLimit = Integer.valueOf(dictData.getDictValue());
            //获取当前日期时间
            LocalDateTime currentDate = LocalDateTime.now();
            // 获取30天前的日期
            LocalDateTime dateBeforeDays = currentDate.minusDays(followArticleLimit);
            // 如果需要特定格式的日期字符串，可以使用DateTimeFormatter
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            String endTime = currentDate.format(formatter);
            String beginTime = dateBeforeDays.format(formatter);
            opmArticle.setBeginTime(beginTime);
            opmArticle.setEndTime(endTime);
            String loginUserId = CyUserUtil.getAuthenBusinessId();
            if (StringUtils.isNotEmpty(opmArticle.getUserId())){
                //设置查询用户 查询某个人的动态列表
                opmArticle.setUserId(opmArticle.getUserId());
            }
            if (StringUtils.isNotEmpty(loginUserId)){
                //设置当前登录人id 用来查询该用户对文章是否已点赞
                opmArticle.setLoginUserId(loginUserId);
            }
            //设置动态的审核状态  只能查看审核通过的
            opmArticle.setExamStatus("1");
            return baseMapper.queryUserArticlePaged(paginationUtility,opmArticle);
        }

        //3.当前登录人的动态
        //如果登录人id和userId相同，证明查看的是自己的动态列表
        String loginUserId = CyUserUtil.getAuthenBusinessId();
        if (StringUtils.isNotEmpty(loginUserId)){
            //查询当前登录人的动态
            opmArticle.setUserId(loginUserId);
        }
        if (StringUtils.isNotEmpty(loginUserId)){
            //设置当前登录人id 用来查询该用户对文章是否已点赞
            opmArticle.setLoginUserId(loginUserId);
        }
        //查看当前登录人的审核状态不限
        return baseMapper.queryUserArticlePaged(paginationUtility,opmArticle);
    }

    /**
    * 查询list opmArticle管理
    * @param opmArticle
    * @return
    */
    @Override
    public List<OpmArticle> findAll(OpmArticle opmArticle){
        return baseMapper.queryOpmArticle(opmArticle);

    }

    /**
    * 导出opmArticle管理
    * @return
    */
    @Override
    public List<OpmArticle> export(OpmArticle opmArticle) {
        List<OpmArticle> opmArticleList = baseMapper.queryOpmArticle(opmArticle);
        return opmArticleList;
    }


    /**
     * 逻辑删除 评论
     * @param dto
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeComment(ArticleCommentDTO dto,String type){
        if (type.equals("app")){
            //app端
            ArticleCommentVO comment = baseMapper.selectCommentById(dto.getBusinessId());
            //删除评论
            int line = baseMapper.deleteComment(dto);
            int childrenCommentCount = 0;
            OpmArticle article = new OpmArticle();
            article.setBusinessId(dto.getArticleId());
            if (dto.getParentId() != null){
                if (comment.getExamStatus().equals("1")){
                    //删除的子评论 需要减掉父评论的数量
                    baseMapper.reduceCommentCount(dto.getParentId());
                    article.setCommentCount(1);
                }
            } else {
                if (comment.getExamStatus().equals("1")){
                    //删除的是父评论 要查父评论下的子评论数量
                    ArticleCommentVO vo = new ArticleCommentVO();
                    vo.setParentId(dto.getBusinessId());
                    Integer count = baseMapper.selectArticleChildrenCommentCount(vo);
                    childrenCommentCount = count;
                    article.setCommentCount(childrenCommentCount + 1);
                }
            }
            if (comment.getExamStatus().equals("1")){
              baseMapper.updateArticleCommentCountById(article);
            }
            return new CyPersistModel(1);
        } else {
            //web端
            ArticleCommentVO comment = baseMapper.selectCommentById(dto.getBusinessId());
            //删除评论
            int line = baseMapper.deleteComment(dto);
            int childrenCommentCount = 0;
            OpmArticle article = new OpmArticle();
            article.setBusinessId(dto.getArticleId());
            if (dto.getParentId() != null){
                if (comment.getExamStatus().equals("1")){
                    //删除的子评论 需要减掉父评论的数量
                    baseMapper.reduceCommentCount(dto.getParentId());
                    article.setCommentCount(1);
                }
            } else {
                if (comment.getExamStatus().equals("1")){
                    //删除的是父评论 要查父评论下的子评论数量
                    ArticleCommentVO vo = new ArticleCommentVO();
                    vo.setParentId(dto.getBusinessId());
                    Integer count = baseMapper.selectArticleChildrenCommentCount(vo);
                    childrenCommentCount = count;
                    article.setCommentCount(childrenCommentCount + 1);
                }
            }
            if (comment.getExamStatus().equals("1")){
                baseMapper.updateArticleCommentCountById(article);
            }
            return new CyPersistModel(1);
        }
    }

    /**
     * 点赞增加点赞数
     * @param articleId
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel likeAdd(Integer articleId,String praiseType){
        String userId = CyUserUtil.getAuthenBusinessId();
        int line = 0;
        if (praiseType.equals("praise")){
            //点赞
            //向点赞表增加一条数据
            LikeDTO likeDTO = new LikeDTO();
            likeDTO.setUserId(CyUserUtil.getAuthenBusinessId());
            likeDTO.setArticleId(articleId);
            //增加主表该动态的点赞数
            baseMapper.addLikeCount(articleId);
            line = baseMapper.likeAdd(likeDTO);
        } else if (praiseType.equals("cancel")){
            //取消点赞
            //删除点赞表中的数据
            baseMapper.likeDelete(userId,articleId);
            //修改主表中该动态的点赞数
            line = baseMapper.reduceLikeCount(articleId);
        } else {
            throw new CyServiceException("点赞失败，请稍后再试");
        }
        return new CyPersistModel(line);
    }

    /**
     * 评论增加点赞数
     * @param commentId
     * @param praiseType  点赞类型 praise:点赞  cancel:取消点赞
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel likeCommentAdd(Integer commentId,String praiseType){
        String userId = CyUserUtil.getAuthenBusinessId();
        int line = 0;
        if (praiseType.equals("praise")){
            //点赞
            //向点赞表增加一条数据
            LikeDTO likeDTO = new LikeDTO();
            likeDTO.setUserId(CyUserUtil.getAuthenBusinessId());
            likeDTO.setCommentId(commentId);
            //增加该条评论的点赞数
            baseMapper.addCommentLikeCount(commentId);
            line = baseMapper.likeAdd(likeDTO);
        } else if (praiseType.equals("cancel")){
            //取消点赞
            //删除点赞表中的数据
            baseMapper.likeDelete(userId,commentId);
            //修改该条评论的点赞数
            line = baseMapper.reduceCommentLikeCount(commentId);
        } else {
            throw new CyServiceException("点赞失败，请稍后再试");
        }
        return new CyPersistModel(line);
    }

    /**
     * 添加评论
     * @param dto
     * @return
     */
    @Override
    public CyPersistModel addComment(ArticleCommentDTO dto) {
        //1、实名判断
        String userId = CyUserUtil.getAuthenBusinessId();
        userUtil.checkUserAuth("请先实名认证!");
        //2、校验 超过规定的时间不能评论 计算天数对应的秒数
        List<DictData> articleConfig = dictionaryService.selectByTypes("article_config");
        DictData dictData = articleConfig.stream().filter(item -> item.getDictLabel().equals("comment_limit_days")).findFirst().orElse(null);
        Integer limitDays = Integer.valueOf(dictData.getDictValue());
        long limitSeconds = limitDays * 24 * 60 * 60;
        //获取当前日期时间
        LocalDateTime now = LocalDateTime.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String nowDate = now.format(formatter);
        //获取动态创建时间
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //根据parentId查询需要回复的评论信息
        ArticleCommentDTO comment = baseMapper.findCommentById(dto.getCommentId());
        String createDate = "";//动态创建时间
        Integer commentedUserId = null;//被评论人id
        if (comment == null){
            //发的是第一级评论
            createDate = sdf.format(new Date());
            commentedUserId = Integer.valueOf(userId);
        } else {
            //发的是二级评论
            createDate = String.valueOf(comment.getCreateDate());
            commentedUserId = comment.getCreateBy();
        }
        try {
            //计算当前时间和动态创建时间的秒数差
            long timeSeconds = TimeUtil.getTimeSecond(createDate,nowDate);
            if (timeSeconds >= limitSeconds){
                //超过限制时间 不允许进行评论
                throw new CyServiceException("该动态超出规定时间,不允许再进行评论");
            }
            log.info("当前时间："+nowDate+"评论时间："+createDate+"相差："+timeSeconds+"秒");
        } catch (ParseException e) {
            e.printStackTrace();
        }
        //3、进行审核   从redis获取动态审核配置  0：自动审核，1：人工审核
        List<DictData> sysConfig = dictionaryService.selectByTypes("sys_config");
        String style = sysConfig.get(0).getDictValue();
        if ("0".equals(style)) {
            //调用百度，自动审核
            boolean flag = true;
            List<String> msg = new ArrayList<>();
            if (StringUtils.isNotBlank(dto.getCommentContent())) {
                CensorResult result = contentCensorService.getCommonTextCensorResult(dto.getCommentContent());
                flag = result.getIsPass();
                if (!flag)
                    msg.addAll(result.getErrorMsg());
            }
            if (flag)
                dto.setExamStatus("1");
            else {
                dto.setExamStatus("3");
                dto.setRemarks(msg.toString());
            }
        } else {
            dto.setExamStatus("0");
        }
        //4、校验通过  进行插入评论数据
        dto.setCommentedUserId(commentedUserId);
        dto.setCreateDate(new Date());
        dto.setUpdateDate(new Date());
        dto.setCreateBy(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        dto.setUpdateBy(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        int line = baseMapper.addComment(dto);
        if (dto.getExamStatus().equals("1")){
            //修改主表的评论数
            baseMapper.addArticleCommentCount(dto.getArticleId());
            //修改子表的评论数 给父级评论数+1
            if (dto.getParentId() != null){
                baseMapper.addCommentCount(dto.getParentId());
            }
        }
        if (dto.getExamStatus().equals("0")){
            //评论未审核通过，修改动态为再次审核状态
            baseMapper.updateArticleExamStatus(dto.getArticleId());
        }
        return new CyPersistModel(1);
    }
}
