package org.rcisoft.business.opmBlacklist.controller;


/*固定导入*/
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.Operation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.exception.CyServiceException;
import jakarta.servlet.http.HttpServletResponse;
import javax.validation.Valid;


import org.rcisoft.business.opmBlacklist.entity.OpmBlackList;
import org.rcisoft.business.opmBlacklist.service.OpmBlackListService;

import java.util.List;

/**
* Created by cy on 2025年2月10日 上午9:46:37.
*/
@RestController
@RequestMapping("/opmBlackList")
public class OpmBlackListController extends CyPaginationController<OpmBlackList> {


    @Autowired
    private OpmBlackListService opmBlackListServiceImpl;

    @PreAuthorize("@cyPerm.hasPerm('sys:blackList:add')")
    @CyOpeLogAnno(title = "system-黑名单表管理-新增黑名单表", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary="添加黑名单表", description="添加黑名单表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid OpmBlackList opmBlackList, BindingResult bindingResult) {
        CyPersistModel data = opmBlackListServiceImpl.persist(opmBlackList);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        opmBlackList);
    }

    @PreAuthorize("@cyPerm.hasPerm('sys:blackList:delete')")
    @CyOpeLogAnno(title = "system-黑名单表管理-删除黑名单表", businessType = CyLogTypeEnum.DELETE)
    @Operation(summary="删除黑名单表", description="删除黑名单表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true,  schema = @Schema(type = "string"))})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId,OpmBlackList opmBlackList) {
        opmBlackList.setBusinessId(businessId);
        CyPersistModel data = opmBlackListServiceImpl.remove(opmBlackList);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    @PreAuthorize("@cyPerm.hasPerm('sys:blackList:update')")
    @CyOpeLogAnno(title = "system-黑名单表管理-修改黑名单表", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary="修改黑名单表", description="修改黑名单表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = false,  schema = @Schema(type = "string"))})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable int businessId, @Valid OpmBlackList opmBlackList, BindingResult bindingResult) {
        opmBlackList.setBusinessId(businessId);
        CyPersistModel data = opmBlackListServiceImpl.merge(opmBlackList);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        opmBlackList);
    }

    @PreAuthorize("@cyPerm.hasPerm('sys:blackList:query')")
    @CyOpeLogAnno(title = "system-黑名单表管理-查询黑名单表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="查询单一黑名单表", description="查询单一黑名单表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true,  schema = @Schema(type = "string"))})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        opmBlackListServiceImpl.findById(businessId));
    }

    @PreAuthorize("@cyPerm.hasPerm('sys:blackList:list')")
    @CyOpeLogAnno(title = "system-黑名单表管理-查询黑名单表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="查询黑名单表集合", description="查询黑名单表集合")
    @GetMapping(value = "/listAll")
    public CyResult listAll(OpmBlackList opmBlackList) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            opmBlackListServiceImpl.findAll(opmBlackList));
    }

    @PreAuthorize("@cyPerm.hasPerm('sys:blackList:list')")
    @CyOpeLogAnno(title = "system-黑名单表管理-查询黑名单表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="分页查询黑名单表集合", description="分页查询黑名单表集合")
    @GetMapping(value = "/list")
    public CyGridModel listByPagination(OpmBlackList opmBlackList) {
        opmBlackListServiceImpl.findAllByPagination(getPaginationUtility(), opmBlackList);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-黑名单表管理-查询黑名单表", businessType = CyLogTypeEnum.EXPORT)
    @Operation(summary = "导出黑名单表信息", description = "导出黑名单表信息")
    @GetMapping(value = "/export")
    public void outOpmBlackList(HttpServletResponse response,OpmBlackList opmBlackList,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="黑名单表信息.xls";break;
            case "1": excelName="黑名单表信息.xlsx";break;
            case "2": excelName="黑名单表信息.csv";break;
        }
        List<OpmBlackList> opmBlackListList = opmBlackListServiceImpl.export(opmBlackList);
        CyEpExcelUtil.exportExcel(opmBlackListList, "黑名单表信息", "黑名单表信息", OpmBlackList.class, excelName, response);
    }
}
